@@ -4,8 +4,10 @@ warnings.filterwarnings("ignore", "apt API not stable yet", FutureWarning)
 import apt
 import apt_pkg
 import os
+import os.path
 import re
 import logging
+import string
 from subprocess import Popen, PIPE
 
 from gettext import gettext as _
@@ -15,19 +17,32 @@ from DistUpgradeView import FuzzyTimeToStr
 # FIXME: we need this only for the later "isinstance()" check
 #        this should probably be solved in some different way
 from DistUpgradeViewText import DistUpgradeViewText
+from DistUpgradeViewNonInteractive import DistUpgradeViewNonInteractive
+
+class CacheException(Exception):
+    pass
+class CacheExceptionLockingFailed(CacheException):
+    pass
 
 class MyCache(apt.Cache):
     # init
-    def __init__(self, config, view, progress=None):
+    def __init__(self, config, view, progress=None, lock=True):
         apt.Cache.__init__(self, progress)
         self.to_install = []
         self.to_remove = []
         self.view = view
         self.config = config
         self.metapkgs = self.config.getlist("Distro","MetaPkgs")
-
+        # acquire lock
+        if lock:
+            try:
+                apt_pkg.PkgSystemLock()
+                self.lock = True
+            except SystemError, e:
+                raise CacheExceptionLockingFailed, e
         # a list of regexp that are not allowed to be removed
         self.removal_blacklist = config.getListFromFile("Distro","RemovalBlacklistFile")
+        self.uname = Popen(["uname","-r"],stdout=PIPE).communicate()[0].strip()
 
     # properties
     @property
@@ -47,6 +62,20 @@ class MyCache(apt.Cache):
         return self._depcache.BrokenCount > 0
 
     # methods
+    def commit(self, fprogress, iprogress):
+        logging.info("cache.commit()")
+        if self.lock:
+            self.releaseLock()
+        apt.Cache.commit(self, fprogress, iprogress)
+
+    def releaseLock(self):
+        if self.lock:
+            try:
+                apt_pkg.PkgSystemUnLock()
+                self.lock = False
+            except SystemError, e:
+                logging.debug("failed to SystemUnLock() (%s) " % e)
+
     def downloadable(self, pkg, useCandidate=True):
         " check if the given pkg can be downloaded "
         if useCandidate:
@@ -77,13 +106,14 @@ class MyCache(apt.Cache):
 
     def restore_snapshot(self):
         """ restore a snapshot """
+        actiongroup = apt_pkg.GetPkgActionGroup(self._depcache)
         self.clear()
         for name in self.to_remove:
             pkg = self[name]
             pkg.markDelete()
         for name in self.to_install:
             pkg = self[name]
-            pkg.markInstall()
+            pkg.markInstall(autoFix=False, autoInst=False)
 
     def sanityCheck(self, view):
         """ check if the cache is ok and if the required metapkgs
@@ -107,6 +137,9 @@ class MyCache(apt.Cache):
         logging.debug("Installing '%s' (%s)" % (pkg, reason))
         if self.has_key(pkg):
             self[pkg].markInstall()
+            if not (self[pkg].markedInstall or self[pkg].markedUpgrade):
+                logging.error("Installing/upgrading '%s' failed" % pkg)
+                #raise (SystemError, "Installing '%s' failed" % pkg)
     def markRemove(self, pkg, reason=""):
         logging.debug("Removing '%s' (%s)" % (pkg, reason))
         if self.has_key(pkg):
@@ -169,6 +202,36 @@ class MyCache(apt.Cache):
         if func is not None:
             func()
 
+    def gutsyQuirks(self):
+        """ this function works around quirks in the feisty->gutsy upgrade """
+        logging.debug("running gutsyQuirks handler")
+        # lowlatency kernel flavour vanished from feisty->gutsy
+        try:
+            (version, build, flavour) = self.uname.split("-")
+            if flavour == 'lowlatency':
+                kernel = "linux-image-generic"
+                if not (self[kernel].isInstalled or self[kernel].markedInstall):
+                    logging.debug("Selecting new kernel '%s'" % kernel)
+                    self[kernel].markInstall()
+        except Exception, e:
+            logging.warning("problem while transitioning lowlatency kernel (%s)" % e)
+        # fix feisty->gutsy utils-linux -> nfs-common transition (LP: #141559)
+        try:
+            for line in map(string.strip, open("/proc/mounts")):
+                if line == '' or line.startswith("#"):
+                    continue
+                try:
+                    (device, mount_point, fstype, options, a, b) = line.split()
+                except Exception, e:
+                    logging.error("can't parse line '%s'" % line)
+                    continue
+                if "nfs" in fstype:
+                    logging.debug("found nfs mount in line '%s', marking nfs-common for install " % line)
+                    self["nfs-common"].markInstall()
+                    break
+        except Exception, e:
+            logging.warning("problem while transitioning util-linux -> nfs-common (%s)" % e)
+
     def feistyQuirks(self):
         """ this function works around quirks in the edgy->feisty upgrade """
         logging.debug("running feistyQuirks handler")
@@ -252,17 +315,56 @@ class MyCache(apt.Cache):
             self.markRemove("nvidia-settings")
             self.markInstall("nvidia-glx")
 
+    def checkThirdPartyQuirks(self):
+        """
+        this function checks for common third party packages and tries
+        to work around issues with them
+        """
+        # envy
+        if (os.path.exists("/var/log/envy-installer.log") and
+            os.path.getsize("/var/log/envy-installer.log") > 0):
+            logging.warning("envy detected, trying to workaround")
+            toDist = self.config.get("Sources","To")
+            pkgs = ["nvidia-glx","nvidia-glx-new","nvidia-glx-legacy"] 
+            # downgrade installed to ubuntu version
+            for name in pkgs:
+                if not self.has_key(name):
+                    continue
+                pkg = self[name]
+                if (pkg.candidateVersion == pkg.installedVersion and
+                    not pkg.candidateDownloadable):
+                    logging.debug("found non-downloadable '%s' " % name)
+                    for ver in pkg._pkg.VersionList:
+                        if ver.FileList:
+                            for (VerFileIter, index) in ver.FileList:
+				indexfile = self._list.FindIndex(VerFileIter)
+                                if (indexfile and 
+                                    indexfile.IsTrusted and
+                                    toDist in indexfile.Describe):
+                                    logging.info("Forcing downgrade of '%s' because of envy" % name)
+                                    self._depcache.SetCandidateVer(pkg._pkg, ver)
+                                    pkg.markInstall()
+                                    break
+        # ...
+        return
+                
+
+
     def checkForKernel(self):
         """ check for the runing kernel and try to ensure that we have
             a updated version
         """
-        uname = Popen(["uname","-r"],stdout=PIPE).communicate()[0].strip()
-        logging.debug("Kernel uname: '%s' " % uname)
+        logging.debug("Kernel uname: '%s' " % self.uname)
         try:
-            (version, build, flavour) = uname.split("-")
+            (version, build, flavour) = self.uname.split("-")
         except Exception, e:
             logging.warning("Can't parse kernel uname: '%s' (self compiled?)" % e)
             return False
+        # now check if we have a SMP system
+        dmesg = Popen(["dmesg"],stdout=PIPE).communicate()[0]
+        if "WARNING: NR_CPUS limit" in dmesg:
+            logging.debug("UP kernel on SMP system!?!")
+            flavour = "generic"
         kernel = "linux-image-%s" % flavour
         if not self.has_key(kernel):
             logging.warning("No kernel: '%s'" % kernel)
@@ -272,11 +374,25 @@ class MyCache(apt.Cache):
             self[kernel].markInstall()
         return True
 
+    def checkPriority(self):
+        # tuple of priorites we require to be installed 
+        need = ('required', )
+        # stuff that its ok not to have
+        removeEssentialOk = self.config.getlist("Distro","RemoveEssentialOk")
+        # check now
+        for pkg in self:
+            if (pkg.candidateDownloadable and
+                not (pkg.isInstalled or pkg.markedInstall) and
+                not pkg.name in removeEssentialOk and
+                pkg.priority in need):
+                self.markInstall(pkg.name, "priority in required set '%s' but not scheduled for install" % need)
+
     def distUpgrade(self, view, serverMode, logfd):
         def _restore_fds(stdout, stderr):
             os.dup2(stdout, 1)
             os.dup2(stderr, 2)
-        text_mode = isinstance(view, DistUpgradeViewText)
+        text_mode = (isinstance(view, DistUpgradeViewText) or
+                     isinstance(view, DistUpgradeViewNonInteractive))
         if text_mode:
             old_stdout = os.dup(1)
             old_stderr = os.dup(2)
@@ -286,6 +402,9 @@ class MyCache(apt.Cache):
             # upgrade (and make sure this way that the cache is ok)
             self.upgrade(True)
 
+            # check that everythink in priority required is installed
+            self.checkPriority()
+
             # see if our KeepInstalled rules are honored
             self.keepInstalledRule()
 
@@ -295,6 +414,9 @@ class MyCache(apt.Cache):
             # check if we got a new kernel
             self.checkForKernel()
 
+            # check for third party stuff (envy)
+            self.checkThirdPartyQuirks()
+
             # install missing meta-packages (if not in server upgrade mode)
             if not serverMode:
                 if not self._installMetaPkgs(view):
@@ -337,7 +459,10 @@ class MyCache(apt.Cache):
                 trusted |= origin.trusted
             if not trusted:
                 untrusted.append(pkg.name)
-        if len(untrusted) > 0:
+        b = self.config.getWithDefault("Distro","AllowUnauthenticated", False)
+        if b:
+            logging.warning("AllowUnauthenticated set!")
+        if (len(untrusted) > 0 and not b):
             untrusted.sort()
             logging.error("Unauthenticated packages found: '%s'" % \
                           " ".join(untrusted))
@@ -473,13 +598,17 @@ class MyCache(apt.Cache):
         return False
 
     def _tryMarkObsoleteForRemoval(self, pkgname, remove_candidates, foreign_pkgs):
-        # sanity check
+        # sanity check, first see if it looks like a runing kernel pkg
+        if pkgname.endswith(self.uname):
+            logging.debug("skipping runing kernel pkg '%s'" % pkgname)
+            return False
         if self._inRemovalBlacklist(pkgname):
             logging.debug("skipping '%s' (in removalBlacklist)" % pkgname)
             return False
         # this is a delete candidate, only actually delete,
         # if it dosn't remove other packages depending on it
         # that are not obsolete as well
+        actiongroup = apt_pkg.GetPkgActionGroup(self._depcache)
         self.create_snapshot()
         try:
             self[pkgname].markDelete()
@@ -504,6 +633,14 @@ class MyCache(apt.Cache):
                     obsolete_pkgs.add(pkg.name)
         return obsolete_pkgs
 
+    def _getUnusedDependencies(self):
+        " get all package names that are not downloadable "
+        unused_dependencies =set()        
+        for pkg in self:
+            if pkg.isInstalled and self._depcache.IsGarbage(pkg._pkg):
+                unused_dependencies.add(pkg.name)
+        return unused_dependencies
+
     def _getForeignPkgs(self, allowed_origin, fromDist, toDist):
         """ get all packages that are installed from a foreign repo
             (and are actually downloadable)
@@ -515,6 +652,7 @@ class MyCache(apt.Cache):
                 # official archive
                 foreign=True
                 for origin in pkg.candidateOrigin:
+                    # FIXME: use some better metric here
                     if fromDist in origin.archive and \
                            origin.origin == allowed_origin:
                         foreign = False
@@ -527,8 +665,12 @@ class MyCache(apt.Cache):
 
 if __name__ == "__main__":
 	import DistUpgradeConfigParser
-	c = MyCache(DistUpgradeConfigParser.DistUpgradeConfig("."))
+        import DistUpgradeView
+	c = MyCache(DistUpgradeConfigParser.DistUpgradeConfig("."),
+                    DistUpgradeView.DistUpgradeView())
 	c.clear()
+        c.create_snapshot()
         c.installedTasks
         c.installTasks(["ubuntu-desktop"])
         print c.getChanges()
+        c.restore_snapshot()
