@@ -96,27 +96,35 @@ class GtkFetchProgressAdapter(apt.progress.FetchProgress):
         self.status = parent.label_status
         self.progress = parent.progressbar_cache
         self.parent = parent
+        self.canceled = False
+        self.button_cancel = parent.button_fetch_cancel
+        self.button_cancel.connect('clicked', self.cancelClicked)
+    def cancelClicked(self, widget):
+        logging.debug("cancelClicked")
+        self.canceled = True
     def mediaChange(self, medium, drive):
-      #print "mediaChange %s %s" % (medium, drive)
-      msg = _("Please insert '%s' into the drive '%s'") % (medium,drive)
-      dialog = gtk.MessageDialog(parent=self.parent.window_main,
-                                 flags=gtk.DIALOG_MODAL,
-                                 type=gtk.MESSAGE_QUESTION,
-                                 buttons=gtk.BUTTONS_OK_CANCEL)
-      dialog.set_markup(msg)
-      res = dialog.run()
-      #print res
-      dialog.destroy()
-      if  res == gtk.RESPONSE_OK:
-        return True
-      return False
+        #print "mediaChange %s %s" % (medium, drive)
+        msg = _("Please insert '%s' into the drive '%s'") % (medium,drive)
+        dialog = gtk.MessageDialog(parent=self.parent.window_main,
+                                   flags=gtk.DIALOG_MODAL,
+                                   type=gtk.MESSAGE_QUESTION,
+                                   buttons=gtk.BUTTONS_OK_CANCEL)
+        dialog.set_markup(msg)
+        res = dialog.run()
+        dialog.destroy()
+        if res == gtk.RESPONSE_OK:
+            return True
+        return False
     def start(self):
-        #self.progress.show()
+        #logging.debug("start")
         self.progress.set_fraction(0)
         self.status.show()
+        self.button_cancel.show()
     def stop(self):
+        #logging.debug("stop")
         self.progress.set_text(" ")
         self.status.set_text(_("Fetching is complete"))
+        self.button_cancel.hide()
     def pulse(self):
         # FIXME: move the status_str and progress_str into python-apt
         # (python-apt need i18n first for this)
@@ -135,7 +143,7 @@ class GtkFetchProgressAdapter(apt.progress.FetchProgress):
 
         while gtk.events_pending():
             gtk.main_iteration()
-        return True
+        return (not self.canceled)
 
 class GtkInstallProgressAdapter(InstallProgress):
     # timeout with no status change when the terminal is expanded
@@ -171,8 +179,8 @@ class GtkInstallProgressAdapter(InstallProgress):
         # if no libgnome2-perl is installed show the terminal
         frontend="gnome"
         if self._cache:
-          if not self._cache.has_key("libgnome2-perl") or \
-             not self._cache["libgnome2-perl"].isInstalled:
+          if (not self._cache.has_key("libgnome2-perl") or 
+              not self._cache["libgnome2-perl"].isInstalled):
             frontend = "dialog"
             self.expander.set_expanded(True)
         self.env = ["VTE_PTY_KEEP_FD=%s"% self.writefd,
@@ -189,8 +197,9 @@ class GtkInstallProgressAdapter(InstallProgress):
         #self.expander_terminal.set_expanded(True)
         self.parent.dialog_error.set_transient_for(self.parent.window_main)
         summary = _("Could not install '%s'") % pkg
-        msg = _("The upgrade aborts now. Please report this bug against the 'update-manager' "
-                "package and include the files in /var/log/dist-upgrade/ in the bugreport.")
+        msg = _("The upgrade will continue but the '%s' package may be "
+                "in a not working state. Please consider submitting a "
+                "bugreport about it.") % pkg
         markup="<big><b>%s</b></big>\n\n%s" % (summary, msg)
         self.parent.dialog_error.realize()
         self.parent.dialog_error.window.set_functions(gtk.gdk.FUNC_MOVE)
@@ -231,6 +240,10 @@ class GtkInstallProgressAdapter(InstallProgress):
     def fork(self):
         pid = self.term.forkpty(envv=self.env)
         if pid == 0:
+          # WORKAROUND for broken feisty vte where envv does not work)
+          for env in self.env:
+            (key, value) = env.split("=")
+            os.environ[key] = value
           # HACK to work around bug in python/vte and unregister the logging
           #      atexit func in the child
           sys.exitfunc = lambda: True
@@ -315,7 +328,8 @@ class DistUpgradeVteTerminal(object):
 
 class DistUpgradeViewGtk(DistUpgradeView,SimpleGladeApp):
     " gtk frontend of the distUpgrade tool "
-    def __init__(self, datadir=None):
+    def __init__(self, datadir=None, logdir=None):
+        self.logdir = logdir
         if not datadir:
           localedir=os.path.join(os.getcwd(),"mo")
           gladedir=os.getcwd()
@@ -410,8 +424,8 @@ class DistUpgradeViewGtk(DistUpgradeView,SimpleGladeApp):
         self._term.connect("contents-changed", self._term_content_changed)
         self._terminal_lines = []
         try:
-          self._terminal_log = open("/var/log/dist-upgrade/term.log","w")
-        except IOError:
+          self._terminal_log = open(os.path.join(self.logdir,"term.log"),"w")
+        except Exception, e:
           # if something goes wrong (permission denied etc), use stdout
           self._terminal_log = sys.stdout
         return self._term
@@ -517,11 +531,17 @@ class DistUpgradeViewGtk(DistUpgradeView,SimpleGladeApp):
         # FIXME: add a whitelist here for packages that we expect to be
         # removed (how to calc this automatically?)
         DistUpgradeView.confirmChanges(self, summary, changes, downloadSize)
+        pkgs_downgrade = len(self.toDowngrade)
         pkgs_remove = len(self.toRemove)
         pkgs_inst = len(self.toInstall)
         pkgs_upgrade = len(self.toUpgrade)
         msg = ""
 
+        if pkgs_downgrade > 0:
+            msg += gettext.ngettext("%d package is going to be downgraded.",
+                                    "%d packages are going to be downgraded.",
+                                    pkgs_downgrade) % pkgs_downgrade
+
         if pkgs_remove > 0:
             # FIXME: make those two seperate lines to make it clear
             #        that the "%" applies to the result of ngettext
@@ -572,15 +592,21 @@ class DistUpgradeViewGtk(DistUpgradeView,SimpleGladeApp):
         self.label_changes.set_markup(msg)
         # fill in the details
         self.details_list.clear()
+        for dg in self.toDowngrade:
+            self.details_list.append([_("<b>Downgrade %s</b>") % dg])
         for rm in self.toRemove:
             self.details_list.append([_("<b>Remove %s</b>") % rm])
         for inst in self.toInstall:
             self.details_list.append([_("Install %s") % inst])
         for up in self.toUpgrade:
             self.details_list.append([_("Upgrade %s") % up])
-        self.treeview_details.scroll_to_cell((0,))
         self.dialog_changes.set_transient_for(self.window_main)
+        # work around problem that scroll_to() does not 
+        # work when it is not realized
         self.dialog_changes.realize()
+        self.treeview_details.realize()
+        self.treeview_details.set_cursor((0,))
+        self.treeview_details.scroll_to_point(0,0)
         self.dialog_changes.window.set_functions(gtk.gdk.FUNC_MOVE)
         res = self.dialog_changes.run()
         self.dialog_changes.hide()
