@@ -24,20 +24,24 @@ from kdeui import *
 from kdecore import *
 from kparts import konsolePart
 from kio import KRun
-from dcopext import DCOPClient, DCOPApp # used to quit adept
+#from dcopext import DCOPClient, DCOPApp # used to quit adept
 
 import sys
 import logging
 import time
 import subprocess
 import traceback
+import tempfile
 
 import apt
 import apt_pkg
 import os
+import shutil
 
 import pty
 
+from DistUpgradeApport import *
+
 from DistUpgradeControler import DistUpgradeControler
 from DistUpgradeView import DistUpgradeView, FuzzyTimeToStr, estimatedDownloadTime, InstallProgress
 from window_main import window_main
@@ -180,10 +184,10 @@ class KDEInstallProgressAdapter(InstallProgress):
         InstallProgress.error(self, pkg, errormsg)
         logging.error("got an error from dpkg for pkg: '%s': '%s'" % (pkg, errormsg))
         summary = _("Could not install '%s'") % pkg
-        msg = _("The upgrade aborts now. Please report this bug against the 'update-manager' "
-                "package and include the files in /var/log/dist-upgrade/ in the bugreport.")
+        msg = _("The upgrade will continue but the '%s' package may be "
+                "in a not working state. Please consider submitting a "
+                "bugreport about it.") % pkg
         msg = "<big><b>%s</b></big><br />%s" % (summary, msg)
-
         dialogue = dialog_error(self.parent.window_main)
         dialogue.label_error.setText(utf8(msg))
         if errormsg != None:
@@ -203,17 +207,21 @@ class KDEInstallProgressAdapter(InstallProgress):
                 "configuration file if you choose to replace it with "
                 "a newer version.")
         markup = "<span weight=\"bold\" size=\"larger\">%s </span> \n\n%s" % (prim, sec)
-        dialogue = dialog_conffile(self.parent.window_main)
-        dialogue.label_conffile.setText(markup)
+        self.confDialogue = dialog_conffile(self.parent.window_main)
+        self.confDialogue.label_conffile.setText(markup)
+        self.confDialogue.textview_conffile.hide()
+        #FIXME, below to be tested
+        #self.confDialogue.resize(self.confDialogue.minimumSizeHint())
+        self.confDialogue.connect(self.confDialogue.show_difference_button, SIGNAL("clicked()"), self.showConffile)
 
         # now get the diff
         if os.path.exists("/usr/bin/diff"):
           cmd = ["/usr/bin/diff", "-u", current, new]
           diff = utf8(subprocess.Popen(cmd, stdout=subprocess.PIPE).communicate()[0])
-          dialogue.textview_conffile.setText(diff)
+          self.confDialogue.textview_conffile.setText(diff)
         else:
-          dialogue.textview_conffile.setText(_("The 'diff' command was not found"))
-        result = dialogue.exec_loop()
+          self.confDialogue.textview_conffile.setText(_("The 'diff' command was not found"))
+        result = self.confDialogue.exec_loop()
         self.time_ui += time.time() - start
         # if replace, send this to the terminal
         if result == QDialog.Accepted:
@@ -221,6 +229,15 @@ class KDEInstallProgressAdapter(InstallProgress):
         else:
             self.parent.konsole.sendInput("n\n")
 
+    def showConffile(self):
+        if self.confDialogue.textview_conffile.isVisible():
+            self.confDialogue.textview_conffile.hide()
+            self.confDialogue.show_difference_button.setText(_("Show Difference >>>"))
+        else:
+            self.confDialogue.textview_conffile.show()
+            self.confDialogue.show_difference_button.setText(_("<<< Hide Difference"))
+       
+
     def fork(self):
         """pty voodoo to attach dpkg's pty to konsole"""
         self.parent.newKonsole()
@@ -315,7 +332,7 @@ class UpgraderMainWindow(window_main):
 
 class DistUpgradeViewKDE(DistUpgradeView):
     """KDE frontend of the distUpgrade tool"""
-    def __init__(self, datadir=None):
+    def __init__(self, datadir=None, logdir=None):
         if not datadir:
           localedir=os.path.join(os.getcwd(),"mo")
         else:
@@ -358,17 +375,28 @@ class DistUpgradeViewKDE(DistUpgradeView):
         self.window_main.showTerminalButton.setEnabled(False)
         self.app.connect(self.window_main.showTerminalButton, SIGNAL("clicked()"), self.showTerminal)
 
-        # create a new DCOP-Client:
-        client = DCOPClient()
-        # connect the client to the local DCOP-server:
-        client.attach()
+        #kdesu requires us to copy the xauthority file before it removes it when Adept is killed
+        copyXauth = tempfile.mktemp("", "adept")
+        if 'XAUTHORITY' in os.environ and os.environ['XAUTHORITY'] != copyXauth:
+            shutil.copy(os.environ['XAUTHORITY'], copyXauth)
+            os.environ["XAUTHORITY"] = copyXauth
 
-        for qcstring_app in client.registeredApplications():
-            app = str(qcstring_app)
-            if app.startswith("adept"): 
-                adept = DCOPApp(qcstring_app, client)
-                adeptInterface = adept.object("MainApplication-Interface")
-                adeptInterface.quit()
+        # Note that with kdesudo this needs --nonewdcop
+        ## create a new DCOP-Client:
+        #client = DCOPClient()
+        ## connect the client to the local DCOP-server:
+        #client.attach()
+
+        #for qcstring_app in client.registeredApplications():
+        #    app = str(qcstring_app)
+        #    if app.startswith("adept"): 
+        #        adept = DCOPApp(qcstring_app, client)
+        #        adeptInterface = adept.object("MainApplication-Interface")
+        #        adeptInterface.quit()
+
+        # This works just as well
+        subprocess.call(["killall", "adept_manager"])
+        subprocess.call(["killall", "adept_updater"])
 
         # init gettext
         gettext.bindtextdomain("update-manager",localedir)
@@ -420,13 +448,21 @@ class DistUpgradeViewKDE(DistUpgradeView):
             issubclass(exctype, SystemExit)):
             return
 
-        tbtext = ''.join(traceback.format_exception(exctype, excvalue, exctb))
-        logging.error("Exception in KDE frontend (invoking crash handler):")
-        logging.error(tbtext)
-        dialog = CrashDialog(self.window_main)
-        dialog.connect(dialog.beastie_url, SIGNAL("leftClickedURL(const QString&)"), self.openURL)
-        dialog.crash_detail.setText(tbtext)
-        dialog.exec_loop()
+        # we handle the exception here, hand it to apport and run the
+        # apport gui manually after it because we kill u-m during the upgrade
+        # to prevent it from poping up for reboot notifications or FF restart
+        # notifications or somesuch
+        lines = traceback.format_exception(exctype, excvalue, exctb)
+        logging.error("not handled expection in KDE frontend:\n%s" % "\n".join(lines))
+        # we can't be sure that apport will run in the middle of a upgrade
+        # so we still show a error message here
+        apport_crash(exctype, excvalue, exctb)
+        if not run_apport():
+            tbtext = ''.join(traceback.format_exception(exctype, excvalue, exctb))
+            dialog = CrashDialog(self.window_main)
+            dialog.connect(dialog.beastie_url, SIGNAL("leftClickedURL(const QString&)"), self.openURL)
+            dialog.crash_detail.setText(tbtext)
+            dialog.exec_loop()
         sys.exit(1)
 
     def openURL(self, url):
@@ -563,9 +599,9 @@ class DistUpgradeViewKDE(DistUpgradeView):
                                     pkgs_upgrade) % pkgs_upgrade
             msg +=" "
         if downloadSize > 0:
-            print "type: " + str(type(apt_pkg.SizeToStr(downloadSize))) + apt_pkg.SizeToStr(downloadSize)
-            print "type: "+ str(type(_("<p>You have to download a total of %s. "))) + _("<p>You have to download a total of %s. ") 
-            print "type: " + str(type(msg))
+            #print "type: " + str(type(apt_pkg.SizeToStr(downloadSize))) + apt_pkg.SizeToStr(downloadSize)
+            #print "type: "+ str(type(_("<p>You have to download a total of %s. "))) + _("<p>You have to download a total of %s. ") 
+            #print "type: " + str(type(msg))
             msg = unicode(msg, 'UTF-8')
             #string = _("<p>You have to download a total of %s. ")
             #msg += string % apt_pkg.SizeToStr(downloadSize)
@@ -628,7 +664,7 @@ class DistUpgradeViewKDE(DistUpgradeView):
         return False
 
     def askYesNoQuestion(self, summary, msg, default='No'):
-        restart = QMessageBox.question(self.window_main, unicode(summary, 'UTF-8'), unicode(msg, 'UTF-8'), QMessageBox.Yes, QMessageBox.No)
+        restart = QMessageBox.question(self.window_main, unicode(summary, 'UTF-8'), unicode("<font>") + unicode(msg, 'UTF-8'), QMessageBox.Yes, QMessageBox.No)
         if restart == QMessageBox.Yes:
             return True
         return False
