@@ -45,8 +45,6 @@ class MetaReleaseCore(object):
     METARELEASE_URI = "http://changelogs.ubuntu.com/meta-release"
     METARELEASE_URI_UNSTABLE = "http://changelogs.ubuntu.com/meta-release-development"
     METARELEASE_URI_PROPOSED = "http://changelogs.ubuntu.com/meta-release-proposed"
-    METARELEASE_FILE = "/var/lib/update-manager/meta-release"
-
     def __init__(self, useDevelopmentRelease=False, useProposed=False):
         # check what uri to use
         if useDevelopmentRelease:
@@ -54,15 +52,7 @@ class MetaReleaseCore(object):
         elif useProposed:
             self.METARELEASE_URI = self.METARELEASE_URI_PROPOSED
 
-        # check if we can write to the global location, if not,
-        # write to homedir
-        try:
-            open(self.METARELEASE_FILE,"a")
-        except IOError, e:
-            path = os.path.expanduser("~/.update-manager-core/")
-            if not os.path.exists(path):
-                os.mkdir(path)
-            self.METARELEASE_FILE = os.path.join(path,"meta-release")
+        self._buildMetaReleaseFile()
         self.metarelease_information = None
         self.downloading = True
         # information about the available dists
@@ -72,6 +62,27 @@ class MetaReleaseCore(object):
         t=thread.start_new_thread(self.download, ())
         #t=thread.start_new_thread(self.check, ())
 
+    def _buildMetaReleaseFile(self):
+        # build the metarelease_file name
+        self.METARELEASE_FILE = os.path.join("/var/lib/update-manager/",
+                                            os.path.basename(self.METARELEASE_URI))
+        # check if we can write to the global location, if not,
+        # write to homedir
+        try:
+            open(self.METARELEASE_FILE,"a")
+        except IOError, e:
+            path = os.path.expanduser("~/.update-manager-core/")
+            if not os.path.exists(path):
+                os.mkdir(path)
+            self.METARELEASE_FILE = os.path.join(path,os.path.basename(self.METARELEASE_URI))
+        # if it is empty, remove it to avoid I-M-S hits on empty file
+        try:
+            if os.path.getsize(self.METARELEASE_FILE) == 0:
+                os.unlink(self.METARELEASE_FILE)
+        except Exception, e:
+            pass
+        return True
+
     def dist_no_longer_supported(self, dist):
         """ virtual function that is called when the distro is no longer
             supported
@@ -154,10 +165,13 @@ class MetaReleaseCore(object):
         #print "download"
         lastmodified = 0
         req = urllib2.Request(self.METARELEASE_URI)
+        # make sure that we always get the latest file (#107716)
+        req.add_header("Cache-Control", "No-Cache")
+        req.add_header("Pragma", "no-cache")
         if os.access(self.METARELEASE_FILE, os.W_OK):
             lastmodified = os.stat(self.METARELEASE_FILE).st_mtime
         if lastmodified > 0:
-            req.add_header("If-Modified-Since", lastmodified)
+            req.add_header("If-Modified-Since", time.asctime(time.gmtime(lastmodified)))
         try:
             uri=urllib2.urlopen(req)
             # sometime there is a root owned meta-relase file
@@ -178,7 +192,7 @@ class MetaReleaseCore(object):
             uri.close()
         except urllib2.URLError:
             if os.path.exists(self.METARELEASE_FILE):
-                f=open(self.METARELEASE_FILE,"r")
+                self.metarelease_information=open(self.METARELEASE_FILE,"r")
         # now check the information we have
         if self.metarelease_information != None:
             self.parse()
