@@ -88,8 +88,8 @@ CHANGELOGS_URI="http://changelogs.ubuntu.com/changelogs/pool/%s/%s/%s/%s_%s/chan
 
 
 class MyCache(apt.Cache):
-    def __init__(self, progress):
-        apt.Cache.__init__(self, progress)
+    def __init__(self, progress, rootdir=None):
+        apt.Cache.__init__(self, progress, rootdir)
         self._initDepCache()
         assert self._depcache.BrokenCount == 0 and self._depcache.DelCount == 0
         self.all_changes = {}
@@ -122,7 +122,33 @@ class MyCache(apt.Cache):
         assert self._depcache.BrokenCount == 0 and self._depcache.DelCount == 0
         self._depcache.Upgrade()
         return wouldDelete
-
+    def matchPackageOrigin(self, pkg, matcher):
+        """ match 'pkg' origin against 'matcher', take versions between
+            installedVersion and candidateVersion into account too
+            Useful if installed pkg A v1.0 is available in both
+            -updates (as v1.2) and -security (v1.1). we want to display
+            it as a security update then
+        """
+        inst_ver = pkg._pkg.CurrentVer
+        cand_ver = self._depcache.GetCandidateVer(pkg._pkg)
+        # init with empty match
+        update_origin = matcher[(None,None)]
+        for ver in pkg._pkg.VersionList:
+            # discard is < than installed ver
+            if (inst_ver and
+                apt_pkg.VersionCompare(ver.VerStr, inst_ver.VerStr) <= 0):
+                #print "skipping '%s' " % ver.VerStr
+                continue
+            # check if we have a match
+            for(verFileIter,index) in ver.FileList:
+                if matcher.has_key((verFileIter.Archive, verFileIter.Origin)):
+                    indexfile = pkg._list.FindIndex(verFileIter)
+                    if indexfile: # and indexfile.IsTrusted:
+                        match = matcher[verFileIter.Archive, verFileIter.Origin]
+                        if match.importance > update_origin.importance:
+                            update_origin = match
+        return update_origin
+        
     def get_changelog(self, name, lock):
         # don't touch the gui in this function, it needs to be thread-safe
         pkg = self[name]
@@ -237,20 +263,24 @@ class UpdateList:
     dist = pipe.read().strip()
     del pipe
     self.distUpgradeWouldDelete = 0
-    
-    templates = [("%s-security" % dist, "Ubuntu", _("Important security updates")
-                                                    , 10),
-                 ("%s-updates" % dist, "Ubuntu", _("Recommended updates"), 9),
-                 ("%s-proposed" % dist, "Ubuntu", _("Proposed updates"), 8),
-                 ("%s-backports" % dist, "Ubuntu", _("Backports"), 7),
-                 (dist, "Ubuntu", _("Distribution updates"), 6)]
-
     self.pkgs = {}
-    self.matcher = {}
     self.num_updates = 0
-    for (origin, archive, desc, importance) in templates:
-        self.matcher[(origin, archive)] = self.UpdateOrigin(desc, importance)
-    self.unknown_origin = self.UpdateOrigin(_("Other updates"), -1)
+    self.matcher = self.initMatcher(dist)
+    
+  def initMatcher(self, dist):
+      # (origin, archive, description, importance)
+      matcher_templates = [
+          ("%s-security" % dist, "Ubuntu", _("Important security updates"),10),
+          ("%s-updates" % dist, "Ubuntu", _("Recommended updates"), 9),
+          ("%s-proposed" % dist, "Ubuntu", _("Proposed updates"), 8),
+          ("%s-backports" % dist, "Ubuntu", _("Backports"), 7),
+          (dist, "Ubuntu", _("Distribution updates"), 6)
+      ]
+      matcher = {}
+      for (origin, archive, desc, importance) in matcher_templates:
+          matcher[(origin, archive)] = self.UpdateOrigin(desc, importance)
+      matcher[(None,None)] = self.UpdateOrigin(_("Other updates"), -1)
+      return matcher
 
   def update(self, cache):
     self.held_back = []
@@ -260,21 +290,14 @@ class UpdateList:
 
     # sort by origin
     for pkg in cache:
-      if pkg.isUpgradable:
+      if pkg.isUpgradable or pkg.markedInstall:
         if pkg.candidateOrigin == None:
             # can happen for e.g. loged packages
             # FIXME: do something more sensible here (but what?)
             print "WARNING: upgradable but no canidateOrigin?!?: ", pkg.name
             continue
-        # TRANSLATORS: updates from an 'unknown' origin
-        originstr = _("Other updates")
-        for aorigin in pkg.candidateOrigin:
-          archive = aorigin.archive
-          origin = aorigin.origin
-        if self.matcher.has_key((archive,origin)) and aorigin.trusted:
-          origin_node = self.matcher[(archive,origin)]
-        else:
-          origin_node = self.unknown_origin
+        # check where the package belongs
+        origin_node = cache.matchPackageOrigin(pkg, self.matcher)
         if not self.pkgs.has_key(origin_node):
           self.pkgs[origin_node] = []
         self.pkgs[origin_node].append(pkg)
@@ -698,6 +721,7 @@ class UpdateManager(SimpleGladeApp):
 
   def on_button_reload_clicked(self, widget):
     #print "on_button_reload_clicked"
+    self.check_metarelease()
     self.invoke_manager(UPDATE)
 
   def on_button_help_clicked(self, widget):
@@ -714,7 +738,7 @@ class UpdateManager(SimpleGladeApp):
     os.environ["APT_LISTCHANGES_FRONTEND"]="none"
 
     # Do not suspend during the update process
-    (dev, cookie) = self.inhibit_sleep()
+    (dev, cookie) = inhibit_sleep()
 
     # set window to insensitive
     self.window_main.set_sensitive(False)
@@ -734,33 +758,10 @@ class UpdateManager(SimpleGladeApp):
 
     # Allow suspend after synaptic is finished
     if cookie != False:
-        self.allow_sleep(dev, cookie)
+        allow_sleep(dev, cookie)
     self.window_main.set_sensitive(True)
     self.window_main.window.set_cursor(None)
 
-
-  def inhibit_sleep(self):
-    """Send a dbus signal to gnome-power-manager to not suspend
-    the system"""
-    try:
-      bus = dbus.Bus(dbus.Bus.TYPE_SESSION)
-      devobj = bus.get_object('org.gnome.PowerManager', 
-                              '/org/gnome/PowerManager')
-      dev = dbus.Interface(devobj, "org.gnome.PowerManager")
-      cookie = dev.Inhibit('UpdateManager', 'Updating system')
-      return (dev, cookie)
-    except Exception, e:
-      print "could not send the dbus Inhibit signal: %s" % e
-      return (False, False)
-
-  def allow_sleep(self, dev, cookie):
-    """Send a dbus signal to gnome-power-manager to allow a suspending
-    the system"""
-    try:
-      dev.UnInhibit(cookie)
-    except Exception, e:
-      print "could not send the dbus UnInhibit signal: %s" % e
-
   def toggled(self, renderer, path):
     """ a toggle button in the listview was toggled """
     iter = self.store.get_iter(path)
@@ -998,17 +999,25 @@ class UpdateManager(SimpleGladeApp):
                    "/usr/share/applications/update-manager.desktop",
                    "--", "/usr/bin/update-manager", "--dist-upgrade")
 
+  def check_metarelease(self):
+      " check for new meta-release information "
+      gconfclient = gconf.client_get_default()
+      self.meta = MetaRelease(self.options.devel_release,
+                              self.options.use_proposed)
+      self.meta.connect("dist_no_longer_supported",self.dist_no_longer_supported)
+      # check if we are interessted in dist-upgrade information
+      # (we are not by default on dapper)
+      if self.options.check_dist_upgrades or \
+             gconfclient.get_bool("/apps/update-manager/check_dist_upgrades"):
+          self.meta.connect("new_dist_available",self.new_dist_available)
+      
+
   def main(self, options):
-    gconfclient = gconf.client_get_default() 
-    self.meta = MetaRelease(options.devel_release, options.use_proposed)
-    self.meta.connect("dist_no_longer_supported",self.dist_no_longer_supported)
+    self.options = options
+
+    # check for new distributin information
+    self.check_metarelease()
 
-    # check if we are interessted in dist-upgrade information
-    # (we are not by default on dapper)
-    if options.check_dist_upgrades or \
-	   gconfclient.get_bool("/apps/update-manager/check_dist_upgrades"):
-      self.meta.connect("new_dist_available",self.new_dist_available)
-    
     while gtk.events_pending():
       gtk.main_iteration()
 
